<!--
 * @Author: liulin liulin19981227@163.com
 * @Date: 2024-03-08 11:20:43
 * @LastEditors: liulin liulin19981227@163.com
 * @LastEditTime: 2024-03-08 15:43:17
 * @Description: 对el-select二次封装，实现选项滚动加载，解决一次性数据过多导致页面卡死问题
-->
<template>
    <el-select
        :value="value"
        v-loadmore="loadMore"
        @focus="focus"
        :filter-method="handleSearch"
        v-bind="$attrs"
        v-on="$listeners"
        :loading="loadingFull"
        style="width: 100%"
        @change="handleChange"
    >
        <el-option
            v-for="option in data"
            :label="option[dictLabel]"
            :value="option[dictValue]"
            :key="option[dictValue]"
        ></el-option>
        <el-option disabled class="loading" value="-1">
            <i v-if="loading" class="el-icon-loading"></i>
            <span v-else>没有更多了</span>
        </el-option>
    </el-select>
</template>
<script>
export default {
    name: 'LoadSelect',
    props: {
        value: {
            type: String,
            default: '',
        },
        // 列表数据
        data: {
            type: Array,
            default: () => [],
        },
        dictLabel: {
            type: String,
            default: 'label',
        },
        dictValue: {
            type: String,
            default: 'value',
        },
        // 调用页数的接口
        request: {
            type: Function,
            default: () => {},
        },
        page: {
            type: [Number, String],
            default: 1,
        },
        // 是否还有更多数据
        isHasMore: {
            type: Boolean,
            default: true
        }
    },
    data() {
        return {
            loading: false,
            keyword: '',
            loadingFull: false,
        }
    },
    methods: {
        // 请求下一页的数据
        loadMore() {
            // 如果没有更多数据，则不请求
            if (!this.isHasMore) {
                return
            }
            // 如果 intercept 属性为 true 则不请求数据
            if (this.loadMore.intercept) {
                return
            }
            this.loadMore.intercept = true
            this.loading = true
            this.request({page: this.page + 1, keyword: this.keyword, more: true}).then(() => { 
                this.loading = false
                // 接口响应之后才把 intercept 设置为 false
                this.loadMore.intercept = false
            })
        },
        // 选中下拉框没有数据时，自动请求第一页的数据
        focus() {
            if (!this.data.length) {
                this.loading = true
                this.request({page: 1}).then(() => {
                    this.loading = false
                })
            }
        },
        // 关键字搜索
        handleSearch(keyword) {
            this.keyword = keyword
            this.loadingFull = true
            this.request({page: 1, keyword}).then(() => {
                this.loadingFull = false
            })
        },
        // 删除选中时，如果请求了关键字，则清除关键字再请求第一页的数据
        clear() {
            if (this.keyword) {
                this.loadingFull = true
                this.keyword = ''
                this.request({page: 1}).then(() => {
                    this.loadingFull = false
                })
            }
        },
        handleChange(id) {
            let data = this.data
            let value = data.filter(i => i.id === id)[0]
            this.$emit('loadSelectChange', value)
        }
    },
}
</script>
<style lang="scss" scoped>
.loading {
    text-align: center;
    color: #fff;
    &:hover {
        color: #21adfd;
    }
    i {
        font-size: 16px;
    }
}
</style>
